# ReaR-specific operations
#
# Copyright 2012 Novell, Inc.
# Author: Charles Arnold <carnold@suse.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import urlgrabber.grabber as grabber

from VMLinux import VMLinux
from exceptions import *
import shutil
import stat
import tempfile
import mount
import msg
import util
import os
import vmdisks

class VMReaR(VMLinux):
    def _kernel(self, base, alt_kernel=None):
        if alt_kernel:
            k=alt_kernel
        else:
            k="kernel"
        return '%s/%s' % (base,k,)
    def _initrd(self, base, alt_initrd=None):
        if alt_initrd:
            i=alt_initrd
        else:
            i="initrd"
        return '%s/%s' % (base,i,)

    def getInstallImagesFromUrl(self, options, url, progress=lambda f:None, supports_32nonpae=False):
        """Retrieves a Xen-enabled ReaR kernel and initrd from the URL, and
        copies them to temporary files.  Returns the (kernel, initrd) tuple of
        filenames."""
        kfd = ifd = kfn = None
        try:
            try:
                if url.startswith('http://') or url.startswith('ftp://'):
                    kfd = grabber.urlopen(self._kernel(url))
                    ifd = grabber.urlopen(self._initrd(url))
                elif url.startswith('nfs:'):
                    mnt = mount.TemporaryMount(url[4:])
                    mountpoint = mnt.mount(mnt)
                    kfd = open(self._kernel(mountpoint), 'r')
                    ifd = open(self._initrd(mountpoint), 'r')
                else:
                    raise InstSrcError(err=InstSrcError.E_UNSUPPORTED, details=url)
                kfn = util.copy_to_temp(kfd, prefix='vmlinuz.', progress=lambda f:progress(f*0.5))
                try:
                    ifn = util.copy_to_temp(ifd, prefix='initrd.', progress=lambda f:progress(0.5+f*0.5))
                except:
                    if kfn:
                        os.unlink(kfn)
                    raise
            except IOError, e:
                raise RuntimeError(msg.invalid_url + ': ' + str(e))
        finally:
            if kfd: kfd.close()
            if ifd: ifd.close()
        return (kfn, ifn)

    def getInstallImagesFromPath(self, path, progress=lambda f:None, supports_32nonpae=False):
        """Retrieves a Xen-enabled ReaR kernel and initrd from the base path, and
        copies them to temporary files.  Returns the (kernel, initrd) tuple of
        filenames."""
        kfd = ifd = kfn = None
        k_xen = i_xen = None
        k_xen = self._kernel(path)
        if os.path.exists(k_xen):
            i_xen = self._initrd(path)
        else:
            k_xen = self._kernel(path,"kernel-xen")
            if not os.path.exists(k_xen):
                k_xen = self._kernel(path,"vmlinuz-xen")
            i_xen = self._initrd(path,"initrd-xen")
        try:
            kfd = open(k_xen, 'r')
            ifd = open(i_xen, 'r')
            kfn = util.copy_to_temp(kfd, prefix='vmlinuz.', progress=lambda f:progress(f*0.5))
            try:
                ifn = util.copy_to_temp(ifd, prefix='initrd.', progress=lambda f:progress(0.5+f*0.5))
            except:
                if kfn:
                    os.unlink(kfn)
                raise
        finally:
            if kfd: kfd.close()
            if ifd: ifd.close()
        return (kfn, ifn)

    def getInstallArgs(self, options, override=None):
        """Return ReaR's kernel arguments for installs."""
        if options.full_virt:
            return ''
        args = VMLinux.getInstallArgs(self, options)
        if options.source:
            if type(options.source) is vmdisks.Disk:
                if options.full_virt:
                    if options.source.get_vdevtype() == 'cdrom' and options.direct_kernel_boot:
                        src = 'cd:/'
                    else:
                        src = 'hd:/dev/%s' % options.source.get_vdev_str()
                else:
                    src = 'hd:/dev/%s' % options.source.get_vdev_str()
            else:
                src = options.source
            args += ' install=%s' % src
        return args

    def preprocessOptions(self, options):
        """Pulls settings out of extra_args."""
        if options.full_virt and (options.os_type == 'rhel4' or options.os_type == 'sles9'):
            options.usb_mouse_abs = False
        if not options.extra_args:
            return
        args = options.extra_args.split()
        extra_args = ''
        for arg in args:
            if arg == 'TERM=xterm':
                # Discard.  Will be re-added later.
                pass
            elif arg.startswith('install='):
                if not options.source:
                    src = arg[8:]
                    if src.startswith('hd:/dev/'):
                        src = 'dev:/' + src[8:]
                    options.source = src
            elif arg.startswith('method='):
                if not options.source:
                    src = arg[7:]
                    if src.startswith('hd://'):
                        src = 'dev:/' + src[5:]
                    options.source = src
            else:
                extra_args += arg + ' '
        options.extra_args = extra_args

    def setupOsSettings(self, guest, progress=lambda f:None):
        """Creates a temporary disk image, in which to pass a autoyast or kickstart
        Text file."""
        progress(1.0)
